﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;

namespace howto_sieve_of_eratosthenes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Make a sieve and display the primes.
        private void btnFindPrimes_Click(object sender, EventArgs e)
        {
            lstPrimes.Items.Clear();
            lblEstPrimes.Text = "";
            lblActualPrimes.Text = "";
            Cursor = Cursors.WaitCursor;
            Refresh();

            // Make the sieve.
            int max = int.Parse(txtMaxNumber.Text);
            bool[] is_prime = MakeSieve(max);

            // Display the primes.
            int num_primes = 0;
            for (int i = 2; i <= max; i++)
                if (is_prime[i])
                {
                    if (num_primes <= 10000) lstPrimes.Items.Add(i);
                    num_primes++;
                }
            if (num_primes > 10000) lstPrimes.Items.Add("...");

            // Display the estimated and actual number of primes.
            lblActualPrimes.Text = num_primes.ToString();

            // Display a Legendre estimate ?(n) = n/(log(n) - 1.08366).
            // See http://mathworld.wolfram.com/PrimeCountingFunction.html.
            double est = (max / (Math.Log(max) - 1.08366));
            lblEstPrimes.Text = est.ToString("0");

            Cursor = Cursors.Default;
        }

        // Build a Sieve of Eratosthenes.
        private bool[] MakeSieve(int max)
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();

            // Make an array indicating whether numbers are prime.
            bool[] is_prime = new bool[max + 1];
            is_prime[2] = true;
            for (int i = 3; i <= max; i += 2) is_prime[i] = true;

            // Cross out multiples of odd primes.
            for (int i = 3; i <= max; i += 2)
            {
                // See if i is prime.
                if (is_prime[i])
                {
                    // Knock out multiples of i.
                    for (int j = i * 3; j <= max; j += i)
                        is_prime[j] = false;
                }
            }

            watch.Stop();
            Console.WriteLine(watch.Elapsed.TotalSeconds.ToString("0.00") + " seconds");

            return is_prime;
        }
    }
}
